@@ -18,7 +18,7 @@ use warnings;
 
 package Clownfish::CFC::Build;
 
-our $VERSION = '0.005001';
+our $VERSION = '0.006000';
 $VERSION = eval $VERSION;
 
 # In order to find Clownfish::CFC::Perl::Build::Charmonic, look in 'lib'
@@ -62,24 +62,45 @@ my $CMARK_SOURCE_DIR = catdir( $MODULES_DIR, 'CommonMark', 'src' );
 my $LEMON_EXE_PATH   = catfile( $LEMON_DIR, "lemon$Config{_exe}" );
 my $PPPORT_H_PATH    = catfile( $INCLUDE,   'ppport.h' );
 
+__PACKAGE__->add_property('with_system_cmark');
+
 sub new {
     my ( $class, %args ) = @_;
-    $args{c_source} = [ $CFC_SOURCE_DIR, $CMARK_SOURCE_DIR ];
+    $args{c_source} = [ $CFC_SOURCE_DIR ];
     $args{include_dirs} ||= [];
     my @aux_include = (
         $INCLUDE,
         $CFC_SOURCE_DIR,
-        $CMARK_SOURCE_DIR,
         curdir(),    # for charmony.h
     );
     push @{ $args{include_dirs} }, @aux_include;
-    return $class->SUPER::new(
+    my $self = $class->SUPER::new(
         %args,
         recursive_test_files => 1,
         charmonizer_params   => {
             charmonizer_c => $CHARMONIZER_C,
         },
     );
+    if ($self->with_system_cmark) {
+        my %pkg_info = eval {
+            require ExtUtils::PkgConfig;
+            ExtUtils::PkgConfig->find('libcmark');
+        };
+        if ($@) {
+            push @{ $self->extra_linker_flags }, '-lcmark';
+        }
+        else {
+            push @{ $self->extra_compiler_flags }, $pkg_info{cflags}
+                if defined $pkg_info{cflags};
+            push @{ $self->extra_linker_flags }, $pkg_info{libs}
+                if defined $pkg_info{libs};
+        }
+    }
+    else {
+        push @{ $self->c_source }, $CMARK_SOURCE_DIR;
+        push @{ $self->include_dirs }, $CMARK_SOURCE_DIR;
+    }
+    return $self;
 }
 
 sub _run_make {
@@ -180,7 +201,7 @@ sub ACTION_code {
 
 sub _valgrind_base_command {
     return
-          "PERL_DESTRUCT_LEVEL=2 LUCY_VALGRIND=1 valgrind "
+          "PERL_DESTRUCT_LEVEL=2 CLOWNFISH_VALGRIND=1 valgrind "
         . "--leak-check=yes "
         . "--show-reachable=yes "
         . "--dsymutil=yes "
@@ -189,8 +210,7 @@ sub _valgrind_base_command {
 
 # Run the entire test suite under Valgrind.
 #
-# For this to work, Lucy must be compiled with the LUCY_VALGRIND environment
-# variable set to a true value, under a debugging Perl.
+# For this to work, the test suite must be run under a debugging Perl.
 #
 # A custom suppressions file will probably be needed -- use your judgment.
 # To pass in one or more local suppressions files, provide a comma separated
@@ -204,9 +224,6 @@ sub ACTION_test_valgrind {
     die "Must be run under a perl that was compiled with -DDEBUGGING"
         unless $self->config('ccflags') =~ /-D?DEBUGGING\b/
                || $^X =~ /\bdebugperl\b/;
-    if ( !$ENV{LUCY_VALGRIND} ) {
-        warn "\$ENV{LUCY_VALGRIND} not true -- possible false positives";
-    }
     $self->depends_on('code');
 
     # Unbuffer STDOUT, grab test file names and suppressions files.
@@ -265,15 +282,20 @@ sub ACTION_dist {
     # the top-level $REPOS_ROOT.  Because some assets we need are outside this
     # directory, we need to copy them in.
     my %to_copy = (
-        '../../CONTRIBUTING.md' => 'CONTRIBUTING.md',
-        '../../LICENSE'         => 'LICENSE',
-        '../../NOTICE'          => 'NOTICE',
-        '../../README.md'       => 'README.md',
-        '../../lemon'           => 'lemon',
-        '../src'                => 'src',
-        '../include'            => 'include',
-        '../modules'            => 'modules',
-        $CHARMONIZER_C          => 'charmonizer.c',
+        '../../CHANGES'          => 'CHANGES',
+        '../../CONTRIBUTING.md'  => 'CONTRIBUTING.md',
+        '../../LICENSE'          => 'LICENSE',
+        '../../NOTICE'           => 'NOTICE',
+        '../../README.md'        => 'README.md',
+        '../../lemon'            => 'lemon',
+        '../src'                 => 'src',
+        '../include'             => 'include',
+        '../modules'             => 'modules',
+        '../common/test/cfbase'  => 't/cfbase',
+        '../common/test/cfclash' => 't/cfclash',
+        '../common/test/cfext'   => 't/cfext',
+        '../common/test/cfinc'   => 't/cfinc',
+        $CHARMONIZER_C           => 'charmonizer.c',
     );
     print "Copying files...\n";
     while (my ($from, $to) = each %to_copy) {
