@@ -17,9 +17,9 @@ use strict;
 use warnings;
 
 package Clownfish::CFC;
-our $VERSION = '0.005001';
+our $VERSION = '0.006000';
 $VERSION = eval $VERSION;
-our $MAJOR_VERSION = 0.005000;
+our $MAJOR_VERSION = 0.006000;
 
 END {
     Clownfish::CFC::Model::Class->_clear_registry();
@@ -27,7 +27,7 @@ END {
 }
 
 use XSLoader;
-BEGIN { XSLoader::load( 'Clownfish::CFC', '0.5.1' ) }
+BEGIN { XSLoader::load( 'Clownfish::CFC', '0.6.0' ) }
 
 {
     package Clownfish::CFC::Util;
@@ -128,22 +128,6 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.5.1' ) }
         exposure          => 'parcel',
     );
 
-    our %fetch_singleton_PARAMS = (
-        parcel     => undef,
-        class_name => undef,
-    );
-
-    sub fetch_singleton {
-        my ( undef, %args ) = @_;
-        verify_args( \%fetch_singleton_PARAMS, %args ) or confess $@;
-        # Maybe prepend parcel prefix.
-        my $parcel = $args{parcel};
-        if ( defined $parcel ) {
-            $parcel = Clownfish::CFC::Model::Parcel->acquire($parcel);
-        }
-        return _fetch_singleton( $parcel, $args{class_name} );
-    }
-
     sub new {
         confess(
             "The constructor for Clownfish::CFC::Model::Class is create()");
@@ -197,14 +181,15 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.5.1' ) }
     our %new_PARAMS = (
         source_dir  => undef,
         path_part   => undef,
-        is_included => 0,
+        ext         => undef,
+        is_included => undef,
     );
 
     sub new {
         my ( $either, %args ) = @_;
         confess "no subclassing allowed" unless $either eq __PACKAGE__;
         verify_args( \%new_PARAMS, %args ) or confess $@;
-        return _new( @args{ qw( source_dir path_part is_included ) } );
+        return _new( @args{ qw( source_dir path_part ext is_included ) } );
     }
 }
 
@@ -252,39 +237,6 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.5.1' ) }
         verify_args( \%new_PARAMS, %args ) or confess $@;
         return _new( @args{qw( dest )} );
     }
-
-    # Recreate host language specific metadata of dependencies.
-    #
-    # TODO: This should be done automatically when building the hierarchy.
-    # But since the current approach relies on runtime introspection, the Perl
-    # modules for all dependencies must first be loaded.
-    sub inherit_metadata {
-        my $self = shift;
-
-        require Clownfish;
-
-        for my $class (@{ $self->ordered_classes }) {
-            next if !$class->included || $class->inert;
-
-            my $class_name = $class->get_name;
-            my $rt_class = Clownfish::Class->fetch_class($class_name)
-                or next;
-
-            for my $rt_method (@{ $rt_class->get_methods }) {
-                if ($rt_method->is_excluded_from_host) {
-                    my $method = $class->method($rt_method->get_name);
-                    $method->exclude_from_host;
-                }
-                else {
-                    my $alias = $rt_method->get_host_alias;
-                    if (defined($alias)) {
-                        my $method = $class->method($rt_method->get_name);
-                        $method->set_host_alias($alias);
-                    }
-                }
-            }
-        }
-    }
 }
 
 {
@@ -344,17 +296,20 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.5.1' ) }
     use Carp;
 
     our %new_PARAMS = (
-        name      => undef,
-        nickname  => undef,
-        version   => undef,
-        file_spec => undef,
+        name          => undef,
+        nickname      => undef,
+        version       => undef,
+        major_version => undef,
+        file_spec     => undef,
     );
 
     sub new {
         my ( $either, %args ) = @_;
         verify_args( \%new_PARAMS, %args ) or confess $@;
         confess "no subclassing allowed" unless $either eq __PACKAGE__;
-        return _new( @args{qw( name nickname version file_spec )} );
+        return _new( @args{qw(
+            name nickname version major_version file_spec
+        )} );
     }
 
     our %new_from_json_PARAMS = (
@@ -370,7 +325,6 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.5.1' ) }
     }
 
     our %new_from_file_PARAMS = (
-        path      => undef,
         file_spec => undef,
     );
 
@@ -378,7 +332,7 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.5.1' ) }
         my ( $either, %args ) = @_;
         verify_args( \%new_from_file_PARAMS, %args ) or confess $@;
         confess "no subclassing allowed" unless $either eq __PACKAGE__;
-        return _new_from_file( @args{qw( path file_spec )} );
+        return _new_from_file( @args{qw( file_spec )} );
     }
 
 #    $parcel = Clownfish::CFC::Model::Parcel->acquire($parcel_name_or_parcel_object);
@@ -735,7 +689,6 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.5.1' ) }
     our %new_PARAMS = (
         hierarchy  => undef,
         lib_dir    => undef,
-        boot_class => undef,
         header     => undef,
         footer     => undef,
     );
@@ -744,7 +697,15 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.5.1' ) }
         my ( $either, %args ) = @_;
         verify_args( \%new_PARAMS, %args ) or confess $@;
         return _new(
-            @args{qw( hierarchy lib_dir boot_class header footer )} );
+            @args{qw( hierarchy lib_dir header footer )} );
+    }
+
+    sub write_bindings {
+        my ( $self, %args ) = @_;
+        $args{parcels} = [ map {
+            Clownfish::CFC::Model::Parcel->acquire($_);
+        } @{ $args{parcels} } ];
+        return $self->_write_bindings( @args{qw( boot_class parcels )} );
     }
 }
 
@@ -762,8 +723,10 @@ BEGIN { XSLoader::load( 'Clownfish::CFC', '0.5.1' ) }
     sub new {
         my ( $either, %args ) = @_;
         verify_args( \%new_PARAMS, %args ) or confess $@;
-        $args{parcel}
-            = Clownfish::CFC::Model::Parcel->acquire( $args{parcel} );
+        if ( exists( $args{parcel} ) ) {
+            $args{parcel}
+                = Clownfish::CFC::Model::Parcel->acquire( $args{parcel} );
+        }
         return _new( @args{qw( parcel class_name )} );
     }
 
