@@ -17,7 +17,6 @@
 #include "charmony.h"
 
 #include <string.h>
-#include <ctype.h>
 
 #include <cmark.h>
 
@@ -72,11 +71,11 @@ S_gen_code_sample(CFCCallable *func, const char *alias, CFCClass *klass,
 
 static char*
 S_gen_positional_sample(const char *prologue, CFCParamList *param_list,
-                        size_t start);
+                        int start);
 
 static char*
 S_gen_labeled_sample(const char *prologue, CFCParamList *param_list,
-                     size_t start);
+                     int start);
 
 static char*
 S_perl_var_name(CFCType *type, int is_ctor_retval);
@@ -423,8 +422,8 @@ S_gen_code_sample(CFCCallable *func, const char *alias, CFCClass *klass,
     prologue = CFCUtil_cat(prologue, "->", alias, NULL);
 
     CFCParamList *param_list = CFCCallable_get_param_list(func);
-    size_t        num_vars   = CFCParamList_num_vars(param_list);
-    size_t        start      = is_constructor ? 0 : 1;
+    int           num_vars   = CFCParamList_num_vars(param_list);
+    int           start      = is_constructor ? 0 : 1;
     char         *sample     = NULL;
 
     if (start == num_vars) {
@@ -444,8 +443,8 @@ S_gen_code_sample(CFCCallable *func, const char *alias, CFCClass *klass,
 
 static char*
 S_gen_positional_sample(const char *prologue, CFCParamList *param_list,
-                        size_t start) {
-    size_t        num_vars = CFCParamList_num_vars(param_list);
+                        int start) {
+    int           num_vars = CFCParamList_num_vars(param_list);
     CFCVariable **vars     = CFCParamList_get_variables(param_list);
     const char  **inits    = CFCParamList_get_initial_values(param_list);
 
@@ -471,15 +470,15 @@ S_gen_positional_sample(const char *prologue, CFCParamList *param_list,
 
 static char*
 S_gen_labeled_sample(const char *prologue, CFCParamList *param_list,
-                     size_t start) {
-    size_t        num_vars = CFCParamList_num_vars(param_list);
+                     int start) {
+    int           num_vars = CFCParamList_num_vars(param_list);
     CFCVariable **vars     = CFCParamList_get_variables(param_list);
     const char  **inits    = CFCParamList_get_initial_values(param_list);
 
     size_t max_name_len = 0;
 
     // Find maximum length of parameter name.
-    for (size_t i = start; i < num_vars; i++) {
+    for (int i = start; i < num_vars; i++) {
         const char *name = CFCVariable_get_name(vars[i]);
         size_t name_len = strlen(name);
         if (name_len > max_name_len) { max_name_len = name_len; }
@@ -487,7 +486,7 @@ S_gen_labeled_sample(const char *prologue, CFCParamList *param_list,
 
     char *params = CFCUtil_strdup("");
 
-    for (size_t i = start; i < num_vars; i++) {
+    for (int i = start; i < num_vars; i++) {
         const char *name    = CFCVariable_get_name(vars[i]);
         const char *init    = inits[i];
         char       *comment = NULL;
@@ -533,7 +532,7 @@ S_perl_var_name(CFCType *type, int is_ctor_retval) {
         }
         else {
             // Skip parcel prefix.
-            if (islower(*specifier)) {
+            if (CFCUtil_islower(*specifier)) {
                 for (specifier++; *specifier; specifier++) {
                     if (*specifier == '_') {
                         specifier++;
@@ -570,21 +569,21 @@ S_camel_to_lower(const char *camel) {
 
     size_t alloc = 1;
     for (size_t i = 1; camel[i]; i++) {
-        if (isupper(camel[i]) && islower(camel[i+1])) {
+        if (CFCUtil_isupper(camel[i]) && CFCUtil_islower(camel[i+1])) {
             alloc += 1;
         }
         alloc += 1;
     }
     char *lower = (char*)MALLOCATE(alloc + 1);
 
-    lower[0] = tolower(camel[0]);
+    lower[0] = CFCUtil_tolower(camel[0]);
     size_t j = 1;
     for (size_t i = 1; camel[i]; i++) {
         // Only insert underscore if next char is lowercase.
-        if (isupper(camel[i]) && islower(camel[i+1])) {
+        if (CFCUtil_isupper(camel[i]) && CFCUtil_islower(camel[i+1])) {
             lower[j++] = '_';
         }
-        lower[j++] = tolower(camel[i]);
+        lower[j++] = CFCUtil_tolower(camel[i]);
     }
     lower[j] = '\0';
 
@@ -824,7 +823,7 @@ S_node_to_pod(cmark_node *node, CFCClass *klass, int header_level) {
                 break;
 
             default:
-                CFCUtil_die("Invalid cmark node type: %d", type);
+                CFCUtil_die("Invalid cmark node type: %d", (int)type);
                 break;
         }
     }
@@ -948,7 +947,7 @@ S_convert_link(cmark_node *link, CFCClass *doc_class, int header_level) {
 
             char *perl_name = CFCUtil_strdup(name);
             for (size_t i = 0; perl_name[i] != '\0'; ++i) {
-                perl_name[i] = tolower(perl_name[i]);
+                perl_name[i] = CFCUtil_tolower(perl_name[i]);
             }
 
             // The Perl POD only contains sections for novel methods. Link
