@@ -16,6 +16,7 @@
 
 #include "charmony.h"
 
+#include <stddef.h>
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -23,12 +24,24 @@
 #include <stdarg.h>
 #include <errno.h>
 #include <sys/stat.h>
+#include <setjmp.h>
 
 // For mkdir.
 #ifdef CHY_HAS_DIRECT_H
   #include <direct.h>
 #endif
 
+#if !defined(CHY_HAS_C99_SNPRINTF) && !defined(CHY_HAS__SCPRINTF)
+  #error "snprintf or replacement not available."
+#endif
+
+/* va_copy is not part of C89. Assume that simple assignment works if it
+ * isn't defined.
+ */
+#ifndef va_copy
+  #define va_copy(dst, src) ((dst) = (src))
+#endif
+
 #ifndef true
     #define true 1
     #define false 0
@@ -36,6 +49,9 @@
 
 #include "CFCUtil.h"
 
+static char    *thrown_error;
+static jmp_buf *current_env;
+
 void
 CFCUtil_null_check(const void *arg, const char *name, const char *file,
                    int line) {
@@ -59,50 +75,36 @@ CFCUtil_strndup(const char *string, size_t len) {
     return copy;
 }
 
-#if defined(CHY_HAS_C99_SNPRINTF) || defined(CHY_HAS__SCPRINTF)
-
 char*
 CFCUtil_sprintf(const char *fmt, ...) {
     va_list args;
 
     va_start(args, fmt);
-#if defined(CHY_HAS_C99_SNPRINTF)
-    int size = vsnprintf(NULL, 0, fmt, args);
-    if (size < 0) { CFCUtil_die("snprintf failed"); }
-#else
-    int size = _vscprintf(fmt, args);
-    if (size < 0) { CFCUtil_die("_scprintf failed"); }
-#endif
-    va_end(args);
-
-    char *string = (char*)MALLOCATE((size_t)size + 1);
-    va_start(args, fmt);
-    vsprintf(string, fmt, args);
+    char *string = CFCUtil_vsprintf(fmt, args);
     va_end(args);
 
     return string;
 }
 
-#elif defined(CHY_HAS__SNPRINTF)
-
 char*
-CFCUtil_sprintf(const char *fmt, ...) {
-    for (size_t size = 32; size * 2 > size; size *= 2) {
-        char *string = (char*)MALLOCATE(size);
-        va_list args;
-        va_start(args, fmt);
-        int result = _vsnprintf(string, size, fmt, args);
-        va_end(args);
-        if (result >= 0 && (size_t)result < size) { return string; }
-        FREEMEM(string);
-    }
-    CFCUtil_die("_snprintf failed");
-    return NULL;
-}
+CFCUtil_vsprintf(const char *fmt, va_list args) {
+    va_list args_copy;
 
+    va_copy(args_copy, args);
+#if defined(CHY_HAS_C99_SNPRINTF)
+    int size = vsnprintf(NULL, 0, fmt, args_copy);
+    if (size < 0) { CFCUtil_die("snprintf failed"); }
 #else
-  #error "snprintf or replacement not available."
+    int size = _vscprintf(fmt, args_copy);
+    if (size < 0) { CFCUtil_die("_scprintf failed"); }
 #endif
+    va_end(args_copy);
+
+    char *string = (char*)MALLOCATE((size_t)size + 1);
+    vsprintf(string, fmt, args);
+
+    return string;
+}
 
 char*
 CFCUtil_cat(char *string, ...) {
@@ -128,13 +130,13 @@ CFCUtil_trim_whitespace(char *text) {
 
     // Find start.
     char *ptr = text;
-    while (*ptr != '\0' && isspace(*ptr)) { ptr++; }
+    while (*ptr != '\0' && CFCUtil_isspace(*ptr)) { ptr++; }
 
     // Find end.
     size_t orig_len = strlen(text);
     char *limit = text + orig_len;
     for (; limit > text; limit--) {
-        if (!isspace(*(limit - 1))) { break; }
+        if (!CFCUtil_isspace(*(limit - 1))) { break; }
     }
 
     // Modify string in place and NULL-terminate.
@@ -154,32 +156,32 @@ CFCUtil_global_replace(const char *string, const char *match,
     int   len_diff        = replacement_len - match_len;
 
     // Allocate space.
-    unsigned count = 0;
+    int count = 0;
     while (NULL != (found = strstr(found, match))) {
         count++;
         found += match_len;
     }
     int size = string_len + count * len_diff + 1;
-    char *modified = (char*)MALLOCATE(size);
+    char *modified = (char*)MALLOCATE((size_t)size);
     modified[size - 1] = 0; // NULL-terminate.
 
     // Iterate through all matches.
     found = (char*)string;
     char *target = modified;
-    size_t last_end = 0;
+    ptrdiff_t last_end = 0;
     if (count) {
         while (NULL != (found = strstr(found, match))) {
-            size_t pos = found - string;
-            size_t unchanged_len = pos - last_end;
+            ptrdiff_t pos = found - string;
+            ptrdiff_t unchanged_len = pos - last_end;
             found += match_len;
-            memcpy(target, string + last_end, unchanged_len);
+            memcpy(target, string + last_end, (size_t)unchanged_len);
             target += unchanged_len;
             last_end = pos + match_len;
-            memcpy(target, replacement, replacement_len);
+            memcpy(target, replacement, (size_t)replacement_len);
             target += replacement_len;
         }
     }
-    size_t remaining = string_len - last_end;
+    size_t remaining = (size_t)(string_len - last_end);
     memcpy(target, string + string_len - remaining, remaining);
 
     return modified;
@@ -204,7 +206,7 @@ CFCUtil_enclose_lines(const char *text, const char *line_prefix,
     while (line_start < text_end) {
         const char *line_end = strchr(line_start, '\n');
         const char *next_start;
-        size_t      line_len;
+        ptrdiff_t   line_len;
 
         if (line_end == NULL) {
             line_len   = text_end - line_start;
@@ -215,8 +217,8 @@ CFCUtil_enclose_lines(const char *text, const char *line_prefix,
             next_start = line_end + 1;
         }
 
-        char *line = (char*)MALLOCATE(line_len + 1);
-        memcpy(line, line_start, line_len);
+        char *line = (char*)MALLOCATE((size_t)line_len + 1);
+        memcpy(line, line_start, (size_t)line_len);
         line[line_len] = '\0';
         result = CFCUtil_cat(result, line_prefix, line, line_postfix, "\n",
                              NULL);
@@ -306,6 +308,55 @@ CFCUtil_wrapped_free(void *ptr) {
     free(ptr);
 }
 
+// Avoid -Wtype-limits warning.
+#if CHAR_MAX <= 127
+  #define IS_ASCII(c) ((c) >= 0)
+#else
+  #define IS_ASCII(c) ((c) >= 0 && (c) <= 127)
+#endif
+
+int
+CFCUtil_isalnum(char c) {
+    return IS_ASCII(c) && isalnum(c);
+}
+
+int
+CFCUtil_isalpha(char c) {
+    return IS_ASCII(c) && isalpha(c);
+}
+
+int
+CFCUtil_isdigit(char c) {
+    return IS_ASCII(c) && isdigit(c);
+}
+
+int
+CFCUtil_islower(char c) {
+    return IS_ASCII(c) && islower(c);
+}
+
+int
+CFCUtil_isspace(char c) {
+    return IS_ASCII(c) && isspace(c);
+}
+
+int
+CFCUtil_isupper(char c) {
+    return IS_ASCII(c) && isupper(c);
+}
+
+char
+CFCUtil_tolower(char c) {
+    if (!IS_ASCII(c)) { return c; }
+    return (char)tolower(c);
+}
+
+char
+CFCUtil_toupper(char c) {
+    if (!IS_ASCII(c)) { return c; }
+    return (char)toupper(c);
+}
+
 int
 CFCUtil_current(const char *orig, const char *dest) {
     // If the destination file doesn't exist, we're not current.
@@ -329,6 +380,15 @@ CFCUtil_current(const char *orig, const char *dest) {
 
 void
 CFCUtil_write_file(const char *filename, const char *content, size_t len) {
+    const char *last_sep = strrchr(filename, CHY_DIR_SEP_CHAR);
+    if (last_sep != NULL && last_sep != filename) {
+        char *dir = CFCUtil_strndup(filename, last_sep - filename);
+        if (!CFCUtil_is_dir(dir) && !CFCUtil_make_path(dir)) {
+            CFCUtil_die("Couldn't create directory '%s'", dir);
+        }
+        FREEMEM(dir);
+    }
+
     FILE *fh = fopen(filename, "w+");
     if (fh == NULL) {
         CFCUtil_die("Couldn't open '%s': %s", filename, strerror(errno));
@@ -352,7 +412,7 @@ CFCUtil_slurp_text(const char *file_path, size_t *len_ptr) {
     }
 
     /* Find length; return NULL if the file has a zero-length. */
-    binary_len = CFCUtil_flength(file);
+    binary_len = (size_t)CFCUtil_flength(file);
     if (binary_len == 0) {
         *len_ptr = 0;
         return NULL;
@@ -604,6 +664,21 @@ CFCUtil_closedir(void *dirhandle, const char *dir) {
 
 /***************************************************************************/
 
+jmp_buf*
+CFCUtil_try_start(jmp_buf *env) {
+    jmp_buf *prev_env = current_env;
+    current_env = env;
+    return prev_env;
+}
+
+char*
+CFCUtil_try_end(jmp_buf *prev_env) {
+    current_env = prev_env;
+    char *error = thrown_error;
+    thrown_error = NULL;
+    return error;
+}
+
 #ifdef CFCPERL
 
 #include "EXTERN.h"
@@ -611,12 +686,37 @@ CFCUtil_closedir(void *dirhandle, const char *dir) {
 #include "XSUB.h"
 #include "ppport.h"
 
+// Undo redefinition by XSUB.h with PERL_IMPLICIT_SYS. Needed for
+// ActivePerl.
+#undef longjmp
+
 void
 CFCUtil_die(const char* format, ...) {
     va_list args;
     va_start(args, format);
-    vcroak(format, &args);
-    va_end(args);
+
+    if (current_env) {
+        thrown_error = CFCUtil_vsprintf(format, args);
+        va_end(args);
+        longjmp(*current_env, 1);
+    }
+    else {
+        vcroak(format, &args);
+        va_end(args);
+    }
+}
+
+void
+CFCUtil_rethrow(char *error) {
+    if (current_env) {
+        thrown_error = error;
+        longjmp(*current_env, 1);
+    }
+    else {
+        sv_setpv(ERRSV, error);
+        FREEMEM(error);
+        croak(NULL);
+    }
 }
 
 void
@@ -633,10 +733,31 @@ void
 CFCUtil_die(const char* format, ...) {
     va_list args;
     va_start(args, format);
-    vfprintf(stderr, format, args);
-    va_end(args);
-    fprintf(stderr, "\n");
-    exit(1);
+
+    if (current_env) {
+        thrown_error = CFCUtil_vsprintf(format, args);
+        va_end(args);
+        longjmp(*current_env, 1);
+    }
+    else {
+        vfprintf(stderr, format, args);
+        va_end(args);
+        fprintf(stderr, "\n");
+        abort();
+    }
+}
+
+void
+CFCUtil_rethrow(char *error) {
+    if (current_env) {
+        thrown_error = error;
+        longjmp(*current_env, 1);
+    }
+    else {
+        fprintf(stderr, "%s\n", error);
+        FREEMEM(error);
+        abort();
+    }
 }
 
 void
